/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache;

import org.jboss.cache.marshall.MethodCall;
import org.jgroups.Address;
import org.jgroups.blocks.RspFilter;

import java.lang.reflect.Method;
import java.util.List;

/**
 * Manager that handles all RPC calls between JBoss Cache instances
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class RPCManagerImpl implements RPCManager
{
   private CacheImpl<?, ?> c;

   /**
    * Empty ctor for mock object creation/unit testing
    */
   public RPCManagerImpl()
   {
   }

   public RPCManagerImpl(CacheSPI c)
   {
      this.c = (CacheImpl) c;
   }

   // for now, we delegate RPC calls to deprecated methods in CacheImpl.

   @SuppressWarnings("deprecation")
   public List callRemoteMethods(List<Address> recipients, MethodCall methodCall, int mode, boolean excludeSelf, long timeout, RspFilter responseFilter) throws Exception
   {
      return c.callRemoteMethods(recipients, methodCall, mode, excludeSelf, timeout, responseFilter);
   }

   @SuppressWarnings("deprecation")
   public List callRemoteMethods(List<Address> recipients, MethodCall methodCall, int mode, boolean excludeSelf, long timeout) throws Exception
   {
      return c.callRemoteMethods(recipients, methodCall, mode, excludeSelf, timeout);
   }

   public boolean isCoordinator()
   {
      return c.isCoordinator();
   }

   public Address getCoordinator()
   {
      return c.getCoordinator();
   }

   @SuppressWarnings("deprecation")
   public List callRemoteMethods(List<Address> recipients, MethodCall methodCall, boolean synchronous, boolean excludeSelf, int timeout) throws Exception
   {
      return c.callRemoteMethods(recipients, methodCall, synchronous, excludeSelf, timeout);
   }

   @SuppressWarnings("deprecation")
   public List callRemoteMethods(List<Address> recipients, Method method, Object[] arguments, boolean synchronous, boolean excludeSelf, long timeout) throws Exception
   {
      return c.callRemoteMethods(recipients, method, arguments, synchronous, excludeSelf, timeout);
   }

   public void setCache(CacheSPI c)
   {
      this.c = (CacheImpl) c;
   }

   /**
    * @return Returns the replication queue (if one is used), null otherwise.
    */
   public ReplicationQueue getReplicationQueue()
   {
      return c.getReplicationQueue();
   }
}
