package org.jboss.cache.interceptors;

import org.jboss.cache.InvocationContext;
import org.jboss.cache.marshall.MethodCall;
import org.jboss.cache.marshall.MethodDeclarations;

/**
 * The interceptor in charge of firing off notifications to cache listeners
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
public class NotificationInterceptor extends BaseTransactionalContextInterceptor
{
   public Object invoke(InvocationContext ctx) throws Throwable
   {
      MethodCall call = ctx.getMethodCall();

      switch (call.getMethodId())
      {
         case MethodDeclarations.blockChannelMethodLocal_id:
            cache.getNotifier().notifyCacheBlocked(cache, true);
            break;
         case MethodDeclarations.unblockChannelMethodLocal_id:
            cache.getNotifier().notifyCacheUnblocked(cache, true);
            break;
         default:
            // do nothing
      }

      // should only kick in as a call returns.
      Object retval = super.invoke(ctx);

      // only invoke on commit.
      switch (call.getMethodId())
      {
         case MethodDeclarations.commitMethod_id:
            // notify commit
            // ctx.is
            cache.getNotifier().notifyTransactionCompleted(ctx.getTransaction(), true, ctx);
            break;
         case MethodDeclarations.rollbackMethod_id:
            // notify rollback
            cache.getNotifier().notifyTransactionCompleted(ctx.getTransaction(), false, ctx);
            break;
         case MethodDeclarations.blockChannelMethodLocal_id:
            cache.getNotifier().notifyCacheBlocked(cache, false);
            break;
         case MethodDeclarations.unblockChannelMethodLocal_id:
            cache.getNotifier().notifyCacheUnblocked(cache, false);
            break;
         default:
            // do nothing;
      }

      return retval;
   }
}
