/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

import org.jboss.cache.config.Configuration;
import org.jboss.cache.factories.XmlConfigurationParser;
import org.jboss.cache.lock.IsolationLevel;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class CacheFactoryTest
{
   Configuration expected;
   String configFile = "META-INF/replSync-service.xml";
   private CacheImpl cache;

   @BeforeMethod(alwaysRun = true)
   public void setUp()
   {
      XmlConfigurationParser parser = new XmlConfigurationParser();
      expected = parser.parseFile(configFile);
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown()
   {
      if (cache != null)
      {
         cache.stop();
      }
   }

   @Test(groups = {"functional"})
   public void testFromConfigFileStarted()
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(configFile);
      // can't test for this anymore since the RuntimeConfig is attached to the running cache
      //assertEquals(expected, cache.getConfiguration());

      assertTrue("Should have started", cache.isStarted());
      doSimpleConfTests(cache.getConfiguration());
   }

   @Test(groups = {"functional"})
   public void testFromConfigFileUnstarted()
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(configFile, false);
      // can't test for this anymore since the RuntimeConfig is attached to the running cache
//      assertEquals(expected, cache.getConfiguration());

      assertFalse("Should not have started", cache.isStarted());

      doSimpleConfTests(cache.getConfiguration());
   }

   @Test(groups = {"functional"})
   public void testFromConfigObjStarted()
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(expected);

      assertTrue("Should have started", cache.isStarted());

      doSimpleConfTests(cache.getConfiguration());
   }

   @Test(groups = {"functional"})
   public void testFromConfigObjUnstarted()
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(expected, false);

      assertFalse("Should not have started", cache.isStarted());

      doSimpleConfTests(cache.getConfiguration());
   }

   private void doSimpleConfTests(Configuration tc)
   {
      assertEquals(Configuration.CacheMode.REPL_SYNC, tc.getCacheMode());
      assertEquals(10000, tc.getLockAcquisitionTimeout());
      assertEquals(IsolationLevel.REPEATABLE_READ, tc.getIsolationLevel());
      assertEquals(true, tc.isUseRegionBasedMarshalling());
      // test some of the XML content.
      // assertEquals("UDP(ip_mcast=true;ip_ttl=64;loopback=false;mcast_addr=228.1.2.3;mcast_port=48866;mcast_recv_buf_size=80000;mcast_send_buf_size=150000;ucast_recv_buf_size=80000;ucast_send_buf_size=150000):PING(down_thread=false;num_initial_members=3;timeout=2000;up_thread=false):MERGE2(max_interval=20000;min_interval=10000):FD_SOCK:VERIFY_SUSPECT(down_thread=false;timeout=1500;up_thread=false):pbcast.NAKACK(down_thread=false;gc_lag=50;max_xmit_size=8192;retransmit_timeout=600,1200,2400,4800;up_thread=false):UNICAST(down_thread=false;min_threshold=10;timeout=600,1200,2400;window_size=100):pbcast.STABLE(desired_avg_gossip=20000;down_thread=false;up_thread=false):FRAG(down_thread=false;frag_size=8192;up_thread=false):pbcast.GMS(join_retry_timeout=2000;join_timeout=5000;print_local_addr=true;shun=true):pbcast.STATE_TRANSFER(down_thread=true;up_thread=true)", tc.getClusterConfig());
   }

   @Test(groups = {"functional"})
   public void testLifecycle() throws Exception
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(expected, false);
      assertFalse(cache.isStarted());
      cache.start();
      assertTrue(cache.isStarted());
      cache.stop();
      assertFalse(cache.isStarted());
   }
}
