/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.buddyreplication;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertNotNull;
import static org.testng.AssertJUnit.assertNull;

import java.util.ArrayList;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.Fqn;
import org.jboss.cache.misc.TestingUtil;

/**
 * Tests how groups are formed and disbanded
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class BuddyAssignmentStateTransferTest extends BuddyReplicationTestsBase
{

   protected int timeout = 10000; // !!!

   protected int getSleepTimeout()
   {
      return timeout;
   }

   public void testNonRegionBasedStateTransfer() throws Exception
   {
      caches = new ArrayList<CacheImpl<Object, Object>>();
      caches.add(createCache(1, "TEST", false, true));

      Fqn<String> main = Fqn.fromString("/a/b/c");
      caches.get(0).put(main, "name", "Joe");

      caches.add(createCache(1, "TEST", false, true));

      TestingUtil.blockUntilViewsReceived(caches.toArray(new CacheImpl[0]), VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      Fqn<String> test = new Fqn<String>(BuddyManager.BUDDY_BACKUP_SUBTREE_FQN,
              BuddyManager.getGroupNameFromAddress(caches.get(0).getLocalAddress()));
      test = new Fqn<String>(test, main);

      assertEquals("State transferred", "Joe", caches.get(1).get(test, "name"));

      caches.add(createCache(1, "TEST", false, true));

      TestingUtil.blockUntilViewsReceived(caches.toArray(new CacheImpl[0]), VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      assertNull("State not transferred", caches.get(2).get(test, "name"));

      // Make 2 the buddy of 0
      caches.get(1).stop();
      caches.set(1, null);

      TestingUtil.sleepThread(getSleepTimeout());

      assertEquals("State transferred", "Joe", caches.get(2).get(test, "name"));
   }

   public void testPersistentStateTransfer() throws Exception
   {
      caches = new ArrayList<CacheImpl<Object, Object>>();

      caches.add(createCacheWithCacheLoader(false, false, false, true, false));
      caches.get(0).getConfiguration().setFetchInMemoryState(false);

      caches.get(0).start();

      Fqn<String> main = Fqn.fromString("/a/b/c");
      caches.get(0).put(main, "name", "Joe");

      caches.add(createCacheWithCacheLoader(false, false, false, true, false));
      caches.get(1).getConfiguration().setFetchInMemoryState(false);

      caches.get(1).start();

      TestingUtil.blockUntilViewsReceived(caches.toArray(new CacheImpl[0]), VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      Fqn test = BuddyManager.getBackupFqn(caches.get(0).getLocalAddress(), main);

      assertFalse("/a/b/c shld not be bin memory", caches.get(1).exists(test));
      assertNotNull("/a/b/c shld be in CL", caches.get(1).getCacheLoader().get(test));
      assertEquals("/a/b/c shld in cache loader", "Joe", caches.get(1).get(test, "name"));
   }
}