/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.buddyreplication;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Cache;
import org.jboss.cache.CacheSPI;
import org.jboss.cache.misc.TestingUtil;

/**
 * Tests how groups are formed and disbanded
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class BuddyGroupAssignmentTest extends BuddyReplicationTestsBase
{
   private Log log = LogFactory.getLog(BuddyGroupAssignmentTest.class);

   public void testSingleBuddy() throws Exception
   {
      log.debug("Running testSingleBuddy");
      caches = createCaches(3, false);

      for (CacheSPI cache : caches)
      {
         printBuddyGroup(cache);
      }

      System.out.println("*** Testing cache 0:");
      assertIsBuddy(caches.get(0), caches.get(1), true);
      System.out.println("*** Testing cache 1:");
      assertIsBuddy(caches.get(1), caches.get(2), true);
      System.out.println("*** Testing cache 2:");
      assertIsBuddy(caches.get(2), caches.get(0), true);
   }

   public void test2Buddies() throws Exception
   {
      log.debug("Running test2Buddies");
      caches = createCaches(2, 3, false);

      TestingUtil.blockUntilViewsReceived(5000, caches.toArray(new Cache[0]));
      TestingUtil.sleepThread(2000);

      System.out.println("*** Testing cache 0");
      assertIsBuddy(caches.get(0), caches.get(1), false);
      assertIsBuddy(caches.get(0), caches.get(2), false);
      System.out.println("*** Testing cache 1");
      assertIsBuddy(caches.get(1), caches.get(2), false);
      assertIsBuddy(caches.get(1), caches.get(0), false);
      System.out.println("*** Testing cache 2");
      assertIsBuddy(caches.get(2), caches.get(1), false);
      assertIsBuddy(caches.get(2), caches.get(0), false);
   }

   public void testRemovalFromClusterSingleBuddy() throws Exception
   {
      log.debug("Running testRemovalFromClusterSingleBuddy");
      caches = createCaches(3, false);

      System.out.println("*** Testing cache 0");
      assertIsBuddy(caches.get(0), caches.get(1), true);
      System.out.println("*** Testing cache 1");
      assertIsBuddy(caches.get(1), caches.get(2), true);
      System.out.println("*** Testing cache 2");
      assertIsBuddy(caches.get(2), caches.get(0), true);

      // now remove a cache from the cluster
      caches.get(1).stop();
      caches.set(1, null);

      TestingUtil.sleepThread(getSleepTimeout());

      // now test new buddy groups
      System.out.println("*** Testing cache 0");
      assertIsBuddy(caches.get(0), caches.get(2), true);
      System.out.println("*** Testing cache 2");
      assertIsBuddy(caches.get(2), caches.get(0), true);
      System.out.println("*** Completed successfully ***");
      assertNoLocks(caches);
   }

   public void testRemovalFromCluster2Buddies() throws Exception
   {
      log.debug("Running testRemovalFromCluster2Buddies");
      caches = createCaches(2, 4, false);
      assertNoLocks(caches);

      System.out.println("*** Testing cache 0");
      assertIsBuddy(caches.get(0), caches.get(1), false);
      assertIsBuddy(caches.get(0), caches.get(2), false);
      System.out.println("*** Testing cache 1");
      assertIsBuddy(caches.get(1), caches.get(2), false);
      assertIsBuddy(caches.get(1), caches.get(3), false);
      System.out.println("*** Testing cache 2");
      assertIsBuddy(caches.get(2), caches.get(3), false);
      assertIsBuddy(caches.get(2), caches.get(0), false);
      System.out.println("*** Testing cache 3");
      assertIsBuddy(caches.get(3), caches.get(0), false);
      assertIsBuddy(caches.get(3), caches.get(1), false);

      // now remove a cache from the cluster
      caches.get(1).stop();
      caches.set(1, null);

      TestingUtil.sleepThread(getSleepTimeout());

      // now test new buddy groups
      System.out.println("*** Testing cache 0");
      assertIsBuddy(caches.get(0), caches.get(2), false);
      assertIsBuddy(caches.get(0), caches.get(3), false);
      System.out.println("*** Testing cache 2");
      assertIsBuddy(caches.get(2), caches.get(3), false);
      assertIsBuddy(caches.get(2), caches.get(0), false);
      System.out.println("*** Testing cache 3");
      assertIsBuddy(caches.get(3), caches.get(0), false);
      assertIsBuddy(caches.get(3), caches.get(2), false);
      assertNoLocks(caches);
   }
}
