package org.jboss.cache.loader;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.config.Configuration;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

/**
 * Created by IntelliJ IDEA.
 * User: bela
 * Date: Jun 9, 2004
 * Time: 9:05:19 AM
 */
public class LocalDelegatingCacheLoaderTest extends CacheLoaderTestsBase
{
   CacheImpl delegating_cache;

   @SuppressWarnings("deprecation")
   protected void configureCache() throws Exception
   {
      delegating_cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      delegating_cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
      delegating_cache.create();
      delegating_cache.start();

      // setCache first ...
      cache.getConfiguration().setCacheLoaderConfig(getSingleCacheLoaderConfig("", "org.jboss.cache.loader.LocalDelegatingCacheLoader", "", false, false, false));

      // force our own cache loader instance
      LocalDelegatingCacheLoader cacheLoader = new LocalDelegatingCacheLoader(delegating_cache);
      cache.setCacheLoader(cacheLoader);
   }

   protected void postConfigure()
   {
      CacheLoader ldr = loader;
      LocalDelegatingCacheLoader ldcl = null;
      do
      {
         if (ldr instanceof LocalDelegatingCacheLoader)
         {
            ldcl = (LocalDelegatingCacheLoader) ldr;
         }
         else
         {
            if (ldr instanceof AbstractDelegatingCacheLoader)
            {
               // look deeper in the delegate chain
               ldr = ((AbstractDelegatingCacheLoader) ldr).getCacheLoader();
            }
            else
            {
               // can't dig any deeper!
               break;
            }
         }
      } while (ldcl == null);

      ldcl.setDelegateCache(delegating_cache);
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
      super.tearDown();
      delegating_cache.stop();
      delegating_cache.destroy();
   }

   @Test(groups = {"functional"}, enabled = false)
   public void testLoadAndStore() throws Exception
   {
      //TODO intentional overload since this test does not pass
      //http://jira.jboss.com/jira/browse/JBCACHE-851
   }

   public void testPartialLoadAndStore()
   {
      // do nothing
   }

   public void testBuddyBackupStore()
   {
      // do nothing
   }

   public void testCacheLoaderThreadSafety()
   {
      // do nothing
   }

   public void testCacheLoaderThreadSafetyMultipleFqns()
   {
      // do nothing
   }
}
