/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.loader;

import static org.testng.AssertJUnit.assertEquals;

import java.util.Iterator;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.interceptors.CacheStoreInterceptor;
import org.jboss.cache.interceptors.Interceptor;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;

/**
 * See http://www.jboss.com/index.html?module=bb&op=viewtopic&p=3919374#3919374
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class SharedCacheLoaderTest extends AbstractCacheLoaderTestBase
{
   private CacheImpl<Object, Object> cache1, cache2;
   private DummyCacheLoader dummyCacheLoader;

   @BeforeMethod(alwaysRun = true)
   @SuppressWarnings("deprecation")
   public void setUp() throws Exception
   {
      if (cache1 != null || cache2 != null) tearDown();

      // set up 2 instances of CacheImpl with shared CacheLoaders.
      cache1 = (CacheImpl<Object, Object>) DefaultCacheFactory.getInstance().createCache(false);
      cache2 = (CacheImpl<Object, Object>) DefaultCacheFactory.getInstance().createCache(false);

      cache1.getConfiguration().setCacheMode("REPL_SYNC");
      cache2.getConfiguration().setCacheMode("REPL_SYNC");

      cache1.getConfiguration().setCacheLoaderConfig(getSingleCacheLoaderConfig("", DummyCacheLoader.class.getName(), "", false, false, true));
      cache2.getConfiguration().setCacheLoaderConfig(getSingleCacheLoaderConfig("", DummyCacheLoader.class.getName(), "", false, false, true));

      cache1.start();
      cache2.start();

      // force setting up the same cache loader class
      dummyCacheLoader = new DummyCacheLoader();

      cache1.setCacheLoader(dummyCacheLoader);
      cache2.setCacheLoader(dummyCacheLoader);
      findCacheStoreInterceptor(cache1).setCache(cache1);
      findCacheStoreInterceptor(cache2).setCache(cache2);
   }

   protected CacheStoreInterceptor findCacheStoreInterceptor(CacheImpl cache)
   {
      Iterator ints = cache.getInterceptors().iterator();
      CacheStoreInterceptor csi = null;
      while (ints.hasNext())
      {
         Interceptor i = (Interceptor) ints.next();
         if (i instanceof CacheStoreInterceptor)
         {
            csi = (CacheStoreInterceptor) i;
            break;
         }
      }
      return csi;
   }

   @AfterMethod(alwaysRun = true)
   protected void tearDown()
   {
      if (cache1 != null) cache1.stop();
      if (cache2 != null) cache2.stop();
      cache1 = null;
      cache2 = null;
   }

   public void testReplicationWithSharedCL() throws Exception
   {
      cache1.put("/test", "one", "two");

      // should have replicated
      assertEquals("two", cache1.get("/test", "one"));
      assertEquals("two", cache2.get("/test", "one"));

      // only a single put() should have happened on the cache loader though.
      assertEquals(1, dummyCacheLoader.getPutCount());
   }
}
