package org.jboss.cache.lock;

import static org.testng.AssertJUnit.assertEquals;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.NodeSPI;
import org.jboss.cache.config.Configuration;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

/**
 * @author Bela Ban
 * @version $Id: AcquireAllTest.java 4444 2007-08-28 14:40:21Z jason.greene@jboss.com $
 */
@Test(groups = {"functional"})
public class AcquireAllTest
{
   CacheImpl cache = null, cache2;
   final Fqn FQN = Fqn.fromString("/myNode");
   final String KEY = "key";
   final String VALUE = "value";

   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
      if (cache != null)
      {
         cache.stop();
         cache.destroy();
         cache = null;
      }
   }


   public void testAcquireAll() throws Exception
   {
      NodeSPI root;
      Object owner = Thread.currentThread();

      cache = createCache(Configuration.CacheMode.LOCAL, IsolationLevel.SERIALIZABLE);
      cache.put("/a/b/c", null);
      cache.put("/1/2/3", null);

      root = cache.getRoot();
      NodeLock lock = root.getLock();

      lock.acquireAll(owner, 2000, NodeLock.LockType.READ);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());

      lock.acquireAll(owner, 2000, NodeLock.LockType.WRITE);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());
   }


   public void testAcquireAllReplicated() throws Exception
   {
      NodeSPI root;
      Object owner = Thread.currentThread();

      cache2 = createCache(Configuration.CacheMode.REPL_ASYNC, IsolationLevel.SERIALIZABLE);
      cache2.put("/a/b/c", null);
      cache2.put("/1/2/3", null);

      cache = createCache(Configuration.CacheMode.REPL_ASYNC, IsolationLevel.SERIALIZABLE);
      root = cache.getRoot();
      NodeLock lock = root.getLock();

      lock.acquireAll(owner, 2000, NodeLock.LockType.READ);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());

      lock.acquireAll(owner, 2000, NodeLock.LockType.WRITE);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());
   }


   private CacheImpl createCache(Configuration.CacheMode mode, IsolationLevel level) throws Exception
   {
      CacheImpl c = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      c.getConfiguration().setCacheMode(mode);
      c.getConfiguration().setIsolationLevel(level);
      c.getConfiguration().setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");
      c.create();
      c.start();
      return c;
   }
}
