package org.jboss.cache.lock;

import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

import javax.transaction.TransactionManager;

import org.jboss.cache.Cache;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.NodeSPI;
import org.jboss.cache.transaction.DummyTransactionManagerLookup;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;
 /**
 * basic locking test
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
@Test(groups = {"functional"})
public class PessimisticLockTest
{
   private Cache<Object, Object> cache;
   private TransactionManager tm;
   private Fqn<String> fqn = Fqn.fromString("/a/b/c");

   @BeforeMethod(alwaysRun = true)
   public void setUp()
   {
      cache = DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setTransactionManagerLookupClass(DummyTransactionManagerLookup.class.getName());
      cache.start();
      tm = cache.getConfiguration().getRuntimeConfig().getTransactionManager();
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown()
   {
      cache.stop();
   }

   public void testPut() throws Exception
   {
      cache.put(fqn, "k", "v");

      tm.begin();
      cache.put(fqn, "k2", "v2");
      NodeSPI<Object, Object> n = (NodeSPI<Object, Object>) cache.getRoot().getChild(fqn);

      assertFalse(n.getLock().isReadLocked());
      assertTrue(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();
   }

   public void testGet() throws Exception
   {
      cache.put(fqn, "k", "v");

      tm.begin();
      cache.get(fqn, "k2");
      NodeSPI<Object, Object> n = (NodeSPI<Object, Object>) cache.getRoot().getChild(fqn);

      assertTrue(n.getLock().isReadLocked());
      assertFalse(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();
   }

   public void testRemove() throws Exception
   {
      cache.put(fqn, "k", "v");
      tm.begin();
      cache.remove(fqn, "k2");
      NodeSPI<Object, Object> n = (NodeSPI<Object, Object>) cache.getRoot().getChild(fqn);

      assertFalse(n.getLock().isReadLocked());
      assertTrue(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();

   }
}
