package org.jboss.cache.lock.pessimistic;

import org.testng.annotations.Test;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.AfterMethod;
import org.jboss.cache.Cache;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.lock.IsolationLevel;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.transaction.DummyTransactionManagerLookup;

import java.util.ArrayList;
import java.util.List;

import javax.transaction.TransactionManager;

@Test(groups = {"functional"}, enabled = false) // Known issue - See JBCACHE-1164 and JBCACHE-1165 
public class ConcurrentPutRemoveTest
{
	private TransactionManager tm;

	private Cache cache;

   @BeforeMethod(alwaysRun = true)
   public void setUp() throws Exception {
		cache = DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
		cache.getConfiguration().setIsolationLevel(IsolationLevel.READ_COMMITTED);
		cache.getConfiguration().setTransactionManagerLookupClass(DummyTransactionManagerLookup.class.getName());
		cache.getConfiguration().setLockAcquisitionTimeout(10000);
		cache.start();
		tm = cache.getConfiguration().getRuntimeConfig().getTransactionManager();
	}

   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
		cache.destroy();
	}

   public void testLock() throws Exception {
		List<SeparateThread> threads = new ArrayList<SeparateThread>();
		for (int x = 0; x < 2; x++) {
			SeparateThread t = new SeparateThread(x);
			threads.add(t);
			t.start();
		}
		for (SeparateThread separateThread : threads) {
			separateThread.join();
			if (separateThread.getException() != null) {
				throw separateThread.getException();
			}
		}

	}

	private class SeparateThread extends Thread {
		Exception e = null;

		private int num = 0;

		public SeparateThread(int num) {
			this.num = num;
		}

		public Exception getException() {
			return e;
		}

		public void run() {
			Thread.currentThread().setName("Thread:" + num);
			try {
				for (int x = 0; x < 1000; x++) {
					tm.begin();
					System.out.println("R" + Thread.currentThread().getName());
					//inside transaction
					cache.removeNode(Fqn.fromString("/a"));
					System.out.println("AR" + Thread.currentThread().getName());
					tm.commit();
					//outside transaction
					System.out.println("P" + Thread.currentThread().getName());
					cache.put(Fqn.fromString("/a/b/c/d"), "text"+x,"b");
					System.out.println("AP" + Thread.currentThread().getName());
				}
			} catch (Exception e) {
				this.e = e;
			}
		}
	}

}
