package org.jboss.cache.lock.pessimistic;

import java.util.ArrayList;
import java.util.List;

import javax.transaction.TransactionManager;

import junit.framework.TestCase;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.transaction.DummyTransactionManager;
import org.jboss.cache.transaction.DummyTransactionManagerLookup;
import org.testng.annotations.Test;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.AfterMethod;

/**
 * Test behaviour of concurent put/remove
 *
 * @author jhalat
 */
@Test(groups = {"functional"}, enabled = false) // Known failure - see JBCACHE-1166 
public class LockUpgradeTest{

   private TransactionManager tm;

   private CacheImpl cache;

   @BeforeMethod(alwaysRun = true)
   protected void setUp() throws Exception {
      Configuration c = new Configuration();
      c.setTransactionManagerLookupClass(DummyTransactionManagerLookup.class.getName());
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(c);
      tm = cache.getConfiguration().getRuntimeConfig().getTransactionManager();
   }

   @AfterMethod(alwaysRun = true)
   protected void tearDown() throws Exception {
      cache.stop();
      cache.destroy();
   }

   public void testLock() throws Exception {
      List<WorkThread> threads = new ArrayList<WorkThread>();
      for (int x = 0; x < 2; x++) {
         WorkThread t = new WorkThread(x == 1);
         threads.add(t);
         t.start();
      }
      for (WorkThread separateThread : threads) {
         separateThread.join();
         if (separateThread.getException() != null) {
            throw separateThread.getException();
         }
      }
   }

   private class WorkThread extends Thread {
      Exception e = null;

      private boolean remove;

      public WorkThread(boolean remove) {
         this.remove = remove;
      }

      public Exception getException() {
         return e;
      }

      public void run() {
         try {
            for (int x = 0; x < 1000; x++) {
               tm.begin();
               if (remove) {
                  cache.remove("/a/b/c/d");
                  cache.remove("/a");
               } else {
                  cache.put("/a/b/c/d", "key","text" + x);
               }
               tm.commit();
               Thread.sleep(1);
            }
         } catch (Exception e) {
            this.e = e;
         }
      }
   }
}
