/*
 *
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.cache.marshall;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

import org.testng.annotations.Test;
/**
 * Checks CRUD methods
 *
 * @author Bela Ban
 * @version $Id: CrudMethodTest.java 4444 2007-08-28 14:40:21Z jason.greene@jboss.com $
 */
@Test(groups = {"functional"})
public class CrudMethodTest
{
   public void testPutMethods()
   {
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putDataEraseMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putDataMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putKeyValMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putDataEraseVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putDataVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putKeyValVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putForExternalReadMethodLocal_id));
      assertTrue(MethodDeclarations.isPutMethod(MethodDeclarations.putForExternalReadVersionedMethodLocal_id));

      assertEquals(8, MethodDeclarations.putMethodIds.size());
   }

   public void testTransactionLifecycleMethods()
   {
      assertTrue(MethodDeclarations.isTransactionLifecycleMethod(MethodDeclarations.commitMethod_id));
      assertTrue(MethodDeclarations.isTransactionLifecycleMethod(MethodDeclarations.rollbackMethod_id));
      assertTrue(MethodDeclarations.isTransactionLifecycleMethod(MethodDeclarations.prepareMethod_id));
      assertTrue(MethodDeclarations.isTransactionLifecycleMethod(MethodDeclarations.optimisticPrepareMethod_id));

      assertEquals(4, MethodDeclarations.transactionLifecycleMethodIds.size());
   }

   public void testCrudMethods()
   {
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putDataEraseMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putDataMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putKeyValMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeDataMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeKeyMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeNodeMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.dataGravitationCleanupMethod_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.moveMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putDataEraseVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putDataVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putKeyValVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeDataVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeKeyVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.removeNodeVersionedMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putForExternalReadMethodLocal_id));
      assertTrue(MethodDeclarations.isCrudMethod(MethodDeclarations.putForExternalReadVersionedMethodLocal_id));

      assertEquals(16, MethodDeclarations.crudMethodIds.size());

      assertFalse(MethodDeclarations.isCrudMethod(MethodDeclarations.getKeyValueMethodLocal_id));
      assertFalse(MethodDeclarations.isCrudMethod(MethodDeclarations.getNodeMethodLocal_id));
      assertFalse(MethodDeclarations.isCrudMethod(MethodDeclarations.getKeysMethodLocal_id));
      assertFalse(MethodDeclarations.isCrudMethod(MethodDeclarations.getChildrenNamesMethodLocal_id));
   }

   public void testBuddyGroupOrganisationMethods()
   {
      assertTrue(MethodDeclarations.isBuddyGroupOrganisationMethod(MethodDeclarations.remoteAnnounceBuddyPoolNameMethod_id));
      assertTrue(MethodDeclarations.isBuddyGroupOrganisationMethod(MethodDeclarations.remoteAssignToBuddyGroupMethod_id));
      assertTrue(MethodDeclarations.isBuddyGroupOrganisationMethod(MethodDeclarations.remoteRemoveFromBuddyGroupMethod_id));

      assertEquals(3, MethodDeclarations.buddyGroupOrganisationMethodIds.size());
   }
}
