package org.jboss.cache.marshall;

import static org.testng.AssertJUnit.assertNotNull;
import static org.testng.AssertJUnit.assertNull;

import java.lang.reflect.Method;
import java.util.Collections;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.buddyreplication.BuddyGroup;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * Tests whether remote calls to RPC methods suppress return values (as sometimes expected)
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
@Test(groups={"functional"})
public class RemoteCallerReturnValuesTest
{
   private CacheImpl<Object, Object> cache;
   private Fqn fqn = Fqn.fromString("/a");
   private Object key = "key";
   private Object value = "value";

   @BeforeMethod(alwaysRun = true)
   public void setUp() throws Exception
   {
      cache = (CacheImpl<Object, Object>) DefaultCacheFactory.getInstance().createCache();
      cache.put(fqn, key, value);
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown()
   {
      cache.stop();
   }

   public void testMethodsThatShouldReturnValues() throws Throwable
   {
      Object retval = cache._replicate(MethodCallFactory.create(MethodDeclarations.clusteredGetMethod, MethodCallFactory.create(MethodDeclarations.getKeyValueMethodLocal, fqn), false));
      assertNotNull(retval);

      assertNotNull(cache._replicate(MethodCallFactory.create(MethodDeclarations.dataGravitationMethod, fqn, false)));
   }


   public void testMethodsThatShouldReturnNull() throws Throwable
   {
      doNullReturnTest(MethodDeclarations.getDataMapMethodLocal, fqn);
      doNullReturnTest(MethodDeclarations.existsMethod, fqn);
      doNullReturnTest(MethodDeclarations.getNodeMethodLocal, fqn);
      doNullReturnTest(MethodDeclarations.getKeysMethodLocal, fqn);
      doNullReturnTest(MethodDeclarations.getChildrenNamesMethodLocal, fqn);
      doNullReturnTest(MethodDeclarations.releaseAllLocksMethodLocal, fqn);
      doNullReturnTest(MethodDeclarations.printMethodLocal, fqn);


      // ------------ buddy replication

      doNullReturnTest(MethodDeclarations.remoteAnnounceBuddyPoolNameMethod, cache.getLocalAddress(), null);
      doNullReturnTest(MethodDeclarations.remoteRemoveFromBuddyGroupMethod, "arse");
      doNullReturnTest(MethodDeclarations.remoteAssignToBuddyGroupMethod, new BuddyGroup(), Collections.emptyMap());

      // ------------ move() api
      doNullReturnTest(MethodDeclarations.moveMethodLocal, fqn, Fqn.ROOT);

      // ------------ Channel BLOCK event
      doNullReturnTest(MethodDeclarations.blockChannelLocal);
      doNullReturnTest(MethodDeclarations.unblockChannelLocal);
   }


   private void doNullReturnTest(Method m, Object... args) throws Throwable
   {
      MethodCall c = MethodCallFactory.create(m, args);
      assertNull(m + " should return a null when called remotely", cache._replicate(c));
   }
}
