/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.marshall;

import static org.testng.AssertJUnit.assertEquals;

import java.io.ObjectInputStream;

import org.jboss.cache.RegionManager;
import org.jboss.cache.Version;
import org.testng.annotations.Test;
/**
 * Tests the enhanced treecache marshaller
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
@Test(groups = {"functional"})
public class VersionAwareMarshallerTest
{
   public void testMarshallerSelection()
   {
      VersionAwareMarshaller marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "2.0.0.GA");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.4.0.GA");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.5.0.GA");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.3.0.GA");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.3.0.SP2");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.3.1.GA");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.2.4.SP2");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());

      marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "1.2.3");
      assertEquals(CacheMarshaller200.class, marshaller.defaultMarshaller.getClass());
   }

   public void testVersionHeaderDefaultCurrent() throws Exception
   {
      VersionAwareMarshaller marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, Version.getVersionString(Version.getVersionShort()));
      byte[] bytes = marshaller.objectToByteBuffer("Hello");

      // expect that this has been serialized using JBoss Serialization so use this to get an OIS.
      ObjectInputStream in = ObjectSerializationFactory.createObjectInputStream(bytes);
      assertEquals("Version header short should be '21'", 21, in.readShort());
   }

   public void testVersionHeader200() throws Exception
   {
      VersionAwareMarshaller marshaller = new VersionAwareMarshaller(new RegionManager(), false, false, "2.0.0.GA");
      byte[] bytes = marshaller.objectToByteBuffer("Hello");

      // expect that this has been serialized using JBoss Serialization so use this to get an OIS.
      ObjectInputStream in = ObjectSerializationFactory.createObjectInputStream(bytes);
      assertEquals("Version header short should be '20'", 20, in.readShort());
   }

}
