package org.jboss.cache.optimistic;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.jboss.cache.CacheSPI;
import org.jboss.cache.InvocationContext;
import org.jboss.cache.interceptors.Interceptor;
import org.jboss.cache.marshall.MethodCall;

/**
 * Handles putXXX() methods: if the given node doesn't exist, it will be created
 * (depending on the create_if_not_exists argument)
 *
 * @author Bela Ban
 * @version $Id: CreateIfNotExistsInterceptor.java,v 1.7 2005/01/26 11:45:14
 *          belaban Exp $
 */
public class MockFailureInterceptor extends Interceptor
{
   public void setCache(CacheSPI cache)
   {
      super.setCache(cache);
   }

   private List<Method> allCalled = new ArrayList<Method>();
   private List failurelist = new ArrayList();

   public Object invoke(InvocationContext ctx) throws Throwable
   {
      MethodCall m = ctx.getMethodCall();
      if (failurelist.contains(m.getMethod()))
      {
         throw new Exception("Failure in method" + m);
      }

      allCalled.add(m.getMethod());

      return null;
   }

   /**
    * @return Returns the failurelist.
    */
   public List getFailurelist()
   {
      return failurelist;
   }

   /**
    * @param failurelist The failurelist to set.
    */
   public void setFailurelist(List failurelist)
   {
      this.failurelist = failurelist;
   }

   /**
    * @return Returns the called.
    */
   public List<Method> getAllCalled()
   {
      return allCalled;
   }

   /**
    * @param called The called to set.
    */
   public void setAllCalled(List<Method> called)
   {
      this.allCalled = called;
   }
}