/*
 * Created on 17-Feb-2005
 *
 *
 *
 */
package org.jboss.cache.optimistic;
import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertNotNull;
import static org.testng.AssertJUnit.assertNull;
import static org.testng.AssertJUnit.assertTrue;

import java.util.HashMap;
import java.util.Map;

import javax.transaction.Transaction;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.Fqn;
import org.jboss.cache.interceptors.Interceptor;
import org.jboss.cache.interceptors.OptimisticCreateIfNotExistsInterceptor;
import org.jboss.cache.interceptors.OptimisticNodeInterceptor;
import org.jboss.cache.loader.SamplePojo;
import org.jboss.cache.transaction.DummyTransactionManager;
import org.jboss.cache.transaction.GlobalTransaction;
import org.jboss.cache.transaction.OptimisticTransactionEntry;
import org.jboss.cache.transaction.TransactionTable;

/**
 * @author xenephon
 */
@SuppressWarnings("unchecked")
public class NodeInterceptorRemoveKeyValTest extends AbstractOptimisticTestCase
{
   public void testTransactionRemoveNoNodeKeyValMethod() throws Exception
   {

      TestListener listener = new TestListener();
      final CacheImpl cache = createCacheWithListener(listener);

      Interceptor interceptor = new OptimisticCreateIfNotExistsInterceptor();
      interceptor.setCache(cache);
      Interceptor nodeInterceptor = new OptimisticNodeInterceptor();
      nodeInterceptor.setCache(cache);
      MockInterceptor dummy = new MockInterceptor();
      dummy.setCache(cache);

      interceptor.setNext(nodeInterceptor);
      nodeInterceptor.setNext(dummy);

      cache.setInterceptorChain(interceptor);

//		 first set up a node with a pojo
      DummyTransactionManager mgr = DummyTransactionManager.getInstance();
      mgr.begin();
      Transaction tx = mgr.getTransaction();

      // inject InvocationContext
      cache.getInvocationContext().setTransaction(tx);
      cache.getInvocationContext().setGlobalTransaction(cache.getCurrentTransaction(tx));

      cache.remove("/one/two", "keyOne");

      assertEquals(null, dummy.getCalled());
      TransactionTable table = cache.getTransactionTable();

      GlobalTransaction gtx = table.get(tx);

      OptimisticTransactionEntry entry = (OptimisticTransactionEntry) table.get(gtx);

      TransactionWorkspace workspace = entry.getTransactionWorkSpace();


      mgr.commit();

      //assert what should be the results of our call
      assertEquals(0, workspace.getNodes().size());
      assertNull(workspace.getNode(Fqn.fromString("/one/two")));
      assertTrue(entry.getLocks().isEmpty());
      assertEquals(1, entry.getModifications().size());
      assertTrue(!cache.exists("/one/two"));
      assertEquals(null, dummy.getCalled());
      cache.stop();
   }

   public void testTransactionRemoveNoKeyValMethod() throws Exception
   {

      TestListener listener = new TestListener();
      final CacheImpl cache = createCacheWithListener(listener);

      Interceptor interceptor = new OptimisticCreateIfNotExistsInterceptor();
      interceptor.setCache(cache);
      Interceptor nodeInterceptor = new OptimisticNodeInterceptor();
      nodeInterceptor.setCache(cache);
      MockInterceptor dummy = new MockInterceptor();
      dummy.setCache(cache);

      interceptor.setNext(nodeInterceptor);
      nodeInterceptor.setNext(dummy);

      cache.setInterceptorChain(interceptor);

//		 first set up a node with a pojo
      DummyTransactionManager mgr = DummyTransactionManager.getInstance();
      mgr.begin();
      Transaction tx = mgr.getTransaction();

      // inject InvocationContext
      cache.getInvocationContext().setTransaction(tx);
      cache.getInvocationContext().setGlobalTransaction(cache.getCurrentTransaction(tx));

      SamplePojo pojo = new SamplePojo(21, "test");
      Map temp = new HashMap();
      temp.put("key1", pojo);
      cache.put("/one/two", temp);

      cache.remove("/one/two", "key2");

      assertEquals(null, dummy.getCalled());
      TransactionTable table = cache.getTransactionTable();

      GlobalTransaction gtx = table.get(tx);

      OptimisticTransactionEntry entry = (OptimisticTransactionEntry) table.get(gtx);

      TransactionWorkspace workspace = entry.getTransactionWorkSpace();


      mgr.commit();

      //assert what should be the results of our call
      assertEquals(3, workspace.getNodes().size());
      assertNotNull(workspace.getNode(Fqn.fromString("/one/two")));
      assertNotNull(workspace.getNode(Fqn.fromString("/one/two")).get("key1"));
      assertNull(workspace.getNode(Fqn.fromString("/one/two")).get("key2"));
      assertTrue(entry.getLocks().isEmpty());
      assertEquals(2, entry.getModifications().size());
      assertTrue(!cache.exists("/one/two"));
      assertEquals(null, dummy.getCalled());
      cache.stop();
   }

   public void testTransactionRemoveKeyValMethod() throws Exception
   {

      TestListener listener = new TestListener();
      final CacheImpl cache = createCacheWithListener(listener);

      Interceptor interceptor = new OptimisticCreateIfNotExistsInterceptor();
      interceptor.setCache(cache);
      Interceptor nodeInterceptor = new OptimisticNodeInterceptor();
      nodeInterceptor.setCache(cache);
      MockInterceptor dummy = new MockInterceptor();
      dummy.setCache(cache);

      interceptor.setNext(nodeInterceptor);
      nodeInterceptor.setNext(dummy);

      cache.setInterceptorChain(interceptor);

//		 first set up a node with a pojo
      DummyTransactionManager mgr = DummyTransactionManager.getInstance();
      mgr.begin();
      Transaction tx = mgr.getTransaction();

      // inject InvocationContext
      cache.getInvocationContext().setTransaction(tx);
      cache.getInvocationContext().setGlobalTransaction(cache.getCurrentTransaction(tx));

      SamplePojo pojo = new SamplePojo(21, "test");
      Map temp = new HashMap();
      temp.put("key1", pojo);
      cache.put("/one/two", temp);

      cache.remove("/one/two", "key1");

      assertEquals(null, dummy.getCalled());
      TransactionTable table = cache.getTransactionTable();

      GlobalTransaction gtx = table.get(tx);

      OptimisticTransactionEntry entry = (OptimisticTransactionEntry) table.get(gtx);

      TransactionWorkspace workspace = entry.getTransactionWorkSpace();


      mgr.commit();

      //assert what should be the results of our call
      assertEquals(3, workspace.getNodes().size());
      assertNotNull(workspace.getNode(Fqn.fromString("/one/two")));
      assertNull(workspace.getNode(Fqn.fromString("/one/two")).get("key1"));
      assertTrue(entry.getLocks().isEmpty());
      assertEquals(2, entry.getModifications().size());
      assertTrue(!cache.exists("/one/two"));
      assertEquals(null, dummy.getCalled());
      cache.stop();
   }
}
