/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.options;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertNull;

import javax.transaction.TransactionManager;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.config.Option;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
@Test(groups = {"functional"})
public class CacheModeLocalSimpleTest
{
   private CacheImpl<Object, Object> cache1, cache2;
   private Option cacheModeLocal;

   @BeforeMethod(alwaysRun = true)
   public void setUp() throws Exception
   {
      cache1 = (CacheImpl<Object, Object>)DefaultCacheFactory.getInstance().createCache(false);
      Configuration c = new Configuration();
      cache1.setConfiguration(c);
      c.setCacheMode("REPL_SYNC");
      c.setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");

      cache2 = (CacheImpl<Object, Object>)DefaultCacheFactory.getInstance().createCache(false);
      c = new Configuration();
      cache2.setConfiguration(c);
      c.setCacheMode("REPL_SYNC");
      c.setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");

      cache1.start();
      cache2.start();

      cacheModeLocal = new Option();
      cacheModeLocal.setCacheModeLocal(true);
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown()
   {
      if (cache1 != null)
      {
         cache1.stop();
         cache1 = null;
      }

      if (cache2 != null)
      {
         cache2.stop();
         cache2 = null;
      }
   }

   public void testCacheModeLocalWithTx() throws Exception
   {
      TransactionManager mgr = cache1.getTransactionManager();
      mgr.begin();

      cache1.put(Fqn.fromString("/replicate"), "k", "v");
      cache1.getInvocationContext().getOptionOverrides().setCacheModeLocal(true);
      cache1.put(Fqn.fromString("/not-replicate"), "k", "v");

      mgr.commit();

      assertEquals("v", cache1.get("/replicate", "k"));
      assertEquals("v", cache1.get("/not-replicate", "k"));

      assertEquals("v", cache2.get("/replicate", "k"));
      assertNull(cache2.get("/not-replicate", "k"));
   }
}
