/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.cache.statetransfer;

import org.jboss.cache.CacheException;
import org.jboss.cache.CacheImpl;
import org.jboss.cache.Version;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.lock.TimeoutException;
import static org.testng.AssertJUnit.fail;
import org.testng.annotations.Test;

import java.io.InputStream;

/**
 * A FailedStateTransferTest.
 *
 * @author Brian Stansberry
 * @version $Revision$
 */
@Test(groups = {"functional"})
public class FailedStateTransferTest extends StateTransferTestBase
{

   public void testFailedStateTransfer() throws Exception
   {
      CacheImpl cache = new SecretiveStateCache();
      cache.setConfiguration(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_ASYNC));
      cache.getConfiguration().setClusterName("VersionedTestBase");
      cache.getConfiguration().setReplVersionString(getReplicationVersion());
      // Use a long timeout to facilitate setting debugger breakpoints
      cache.getConfiguration().setStateRetrievalTimeout(60000);

      // Put the cache in the map before starting, so if it fails in
      // start it can still be destroyed later
      caches.put("secretive", cache);

      cache.create();
      cache.start();


      CacheImpl recipient = new SecretiveStateCache();
      recipient.setConfiguration(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_ASYNC));
      recipient.getConfiguration().setClusterName("VersionedTestBase");
      recipient.getConfiguration().setReplVersionString(getReplicationVersion());
      // Use a long timeout to facilitate setting debugger breakpoints
      recipient.getConfiguration().setStateRetrievalTimeout(60000);

      //Put the cache in the map before starting, so if it fails in
      // start it can still be destroyed later
      caches.put("secretive2", recipient);

      try
      {
         recipient.create();
         recipient.start();
         fail("start() should throw an exception");
      }
      catch (CacheException good)
      {
         // this is what we want
      }
   }

   protected String getReplicationVersion()
   {
      return Version.version;
   }

   private static class SecretiveStateCache extends CacheImpl<Object, Object>
   {
      SecretiveStateCache() throws Exception
      {
         super();
         setMessageListener(new Adaptor());
      }

      class Adaptor extends MessageListenerAdaptor
      {

         @Override
         public void setState(byte[] new_state)
         {
            setStateException = new TimeoutException("Planned Timeout");
         }

         @Override
         public void setState(InputStream istream)
         {
            setStateException = new TimeoutException("Planned Timeout");
         }

         @Override
         public void setState(String state_id, byte[] state)
         {
            setStateException = new TimeoutException("Planned Timeout");
         }

         @Override
         public void setState(String state_id, InputStream istream)
         {
            setStateException = new TimeoutException("Planned Timeout");
         }
      }
   }
}
